package net.rzdata.mcp.server.crm;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson2.JSONObject;
import net.rzdata.mcp.server.crm.config.CrmProperties;
import org.springframework.ai.tool.annotation.Tool;
import org.springframework.ai.tool.annotation.ToolParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

@Service
public class CrmMcpService {
    @Autowired
    private CrmProperties crmProperties;

    /**
     * 公共的HTTP请求方法
     */
    private String executeRequest(String apiPath, JSONObject requestBody) {
        return HttpRequest.post(crmProperties.getApiUrl(apiPath))
                .header("tenantid", crmProperties.getTenantId())
                .header("Authorization", crmProperties.getToken())
                .body(requestBody.toString())
                .execute()
                .body();
    }


    @Tool(description = "根据客户名称或客户编码查询客户负责人。clientCode和clientName必须二选一。如果都提取不到则输出：请提供需要查询的客户。如果提取到多个值则输出：目前只能查询单个客户数据")
    public String getClientResponsible(
            @ToolParam(description = "客户编码，一般类似于'XX.XX.XXXX.XXX'格式，例如：01.01.0023.024。与clientName必须二选一", required = false) String clientCode,
            @ToolParam(description = "客户名称，一般类似于'某某课题组'或'某某公司'或'某某实验室'或'某某经营部'或'某某研究院'或'某某研究所'或'某某测试'等组织，例如：张三课题组，广州小懒科技有限公司，金域测试、金域课题组111、玉尘测试A01、睿展小组3号。与clientCode必须二选一", required = false) String clientName

    ) {
        Assert.isTrue(StrUtil.isNotEmpty(clientCode) || StrUtil.isNotEmpty(clientName),
                "客户编码和客户名称不能同时为空");

        JSONObject requestBody = new JSONObject();
        requestBody.put("clientCode", clientCode);
        requestBody.put("clientName", clientName);

        return executeRequest("ai/query/getClientResponsible", requestBody);
    }

    @Tool(description = "根据订单编码（即订单号）查询订单负责人。orderCode是必填项，如果提取不到则输出：请告知需要查询的订单号")
    public String getOrderSaller(
            @ToolParam(description = "订单编码（即订单号），值一般是由字母、数字和'-'组成，特征一般类似于'NJ00027202506090001'，或'NJ00027202506090001-G'或'NJ00027202506090001-T-1'，或'dev-NJ00027202506090001'") String orderCode
    ) {
        Assert.isTrue(StrUtil.isNotEmpty(orderCode), "订单编码不能为空");

        JSONObject requestBody = new JSONObject();
        requestBody.put("orderCode", orderCode);

        return executeRequest("ai/query/getOrderSaller", requestBody);
    }

    @Tool(description = "根据客户名称或客户编码查询某个时间段内的订单销售额。clientCode和clientName必须二选一。如果都提取不到则输出：请提供需要查询的客户。如果提取到多个值则输出：目前只能查询单个客户数据。对于'去年'、'今年'这种时间词，需要结合当前系统时间来判断")
    public String sumOrderTotalForClientInTime(
            @ToolParam(description = "客户编码，一般类似于'XX.XX.XXXX.XXX'格式，例如：01.01.0023.024。与clientName必须二选一", required = false) String clientCode,
            @ToolParam(description = "客户名称，一般类似于'某某课题组'或'某某公司'或'某某实验室'或'某某经营部'或'某某研究院'或'某某研究所'或'某某测试'等组织，例如：张三课题组，广州小懒科技有限公司，金域测试、金域课题组111、玉尘测试A01、睿展小组3号。与clientCode必须二选一", required = false) String clientName,
            @ToolParam(description = "开始时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String startTime,
            @ToolParam(description = "结束时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String endTime
    ) {

        JSONObject requestBody = new JSONObject();
        requestBody.put("clientCode", clientCode);
        requestBody.put("clientName", clientName);
        requestBody.put("startTime", startTime);
        requestBody.put("endTime", endTime);

        return executeRequest("ai/query/sumOrderTotalForClientInTime", requestBody);
    }

    @Tool(description = "根据客户名称或客户编码查询某个时间段内购买的某个产品货号的数量。clientCode和clientName必须二选一。如果都提取不到则输出：请提供需要查询的客户。productnum必填，如果提取不到则输出：请提供需要查询的货号。startTime和endTime必填，如果提取不到则输出：请提供需要查询的采购时间范围")
    public String sumProductNumForClientInTime(
            @ToolParam(description = "客户编码，一般类似于'XX.XX.XXXX.XXX'格式，例如：01.01.0023.024。与clientName必须二选一", required = false) String clientCode,
            @ToolParam(description = "客户名称，一般类似于'某某课题组'或'某某公司'或'某某实验室'或'某某经营部'或'某某研究院'或'某某研究所'等组织，例如：张三课题组，广州小懒科技有限公司。与clientCode必须二选一", required = false) String clientName,
            @ToolParam(description = "产品货号，一般类似于'XXXX-XX'格式，例如：C112-01。必填参数", required = false) String productnum,
            @ToolParam(description = "采购开始时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。必填参数", required = false) String startTime,
            @ToolParam(description = "采购结束时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。必填参数", required = false) String endTime
    ) {
        JSONObject requestBody = new JSONObject();
        requestBody.put("clientCode", clientCode);
        requestBody.put("clientName", clientName);
        requestBody.put("productnum", productnum);
        requestBody.put("startTime", startTime);
        requestBody.put("endTime", endTime);

        return executeRequest("ai/query/sumProductNumForClientInTime", requestBody);
    }

    @Tool(description = "根据订单编码（即订单号）查询订单购买的产品明细。orderCode是必填项，如果提取不到则输出：请告知需要查询的订单号")
    public String detailOrderProduct(
            @ToolParam(description = "订单编码（即订单号），值一般是由字母、数字和'-'组成，特征一般类似于'NJ00027202506090001'，或'NJ00027202506090001-G'或'NJ00027202506090001-T-1'，或'dev-NJ00027202506090001'") String orderCode
    ) {
        Assert.isTrue(StrUtil.isNotEmpty(orderCode), "订单编码不能为空");

        JSONObject requestBody = new JSONObject();
        requestBody.put("orderCode", orderCode);

        return executeRequest("ai/query/detailOrderProduct", requestBody);
    }

    @Tool(description = "根据客户名称或客户编码查询某个时间段内购买的产品主货号列表")
    public String listProductForClientInTime(
            @ToolParam(description = "客户编码，一般类似于'XX.XX.XXXX.XXX'格式，例如：01.01.0023.024。与clientName必须二选一", required = false) String clientCode,
            @ToolParam(description = "客户名称，一般类似于'某某课题组'或'某某公司'或'某某实验室'或'某某经营部'或'某某研究院'或'某某研究所'或'某某测试'等组织，例如：张三课题组，广州小懒科技有限公司，金域测试、金域课题组111、玉尘测试A01、睿展小组3号。与clientCode必须二选一", required = false) String clientName,
            @ToolParam(description = "开始时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String startTime,
            @ToolParam(description = "结束时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String endTime
    ) {
        JSONObject requestBody = new JSONObject();
        requestBody.put("clientCode", clientCode);
        requestBody.put("clientName", clientName);
        requestBody.put("startTime", startTime);
        requestBody.put("endTime", endTime);

        return executeRequest("ai/query/listProductForClientInTime", requestBody);
    }

    @Tool(description = "查询客户在某个时间段内的欠款额")
    public String getUnpaidAmountForClient(
            @ToolParam(description = "客户编码，一般类似于'XX.XX.XXXX.XXX'格式，例如：01.01.0023.024。与clientName必须二选一", required = false) String clientCode,
            @ToolParam(description = "客户名称，一般类似于'某某课题组'或'某某公司'或'某某实验室'或'某某经营部'或'某某研究院'或'某某研究所'或'某某测试'等组织，例如：张三课题组，广州小懒科技有限公司，金域测试、金域课题组111、玉尘测试A01、睿展小组3号。与clientCode必须二选一", required = false) String clientName,
            @ToolParam(description = "开始时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String startTime,
            @ToolParam(description = "结束时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String endTime
    ) {
        JSONObject requestBody = new JSONObject();
        requestBody.put("clientCode", clientCode);
        requestBody.put("clientName", clientName);
        requestBody.put("startTime", startTime);
        requestBody.put("endTime", endTime);

        return executeRequest("ai/query/getUnpaidAmountForClient", requestBody);
    }

    @Tool(description = "根据主货号查询某个时间段内购买过该产品的客户名称列表")
    public String listClientsByMainNumInTime(
            @ToolParam(description = "主货号，一般类似于'XXXX-XXXX'格式，例如：1234-5678", required = false) String mainNum,
            @ToolParam(description = "开始时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String startTime,
            @ToolParam(description = "结束时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String endTime
    ) {
        JSONObject requestBody = new JSONObject();
        requestBody.put("mainNum", mainNum);
        requestBody.put("startTime", startTime);
        requestBody.put("endTime", endTime);

        return executeRequest("ai/query/listClientsByMainNumInTime", requestBody);
    }

    @Tool(description = "根据客户编码或客户名称查询时间区间内的订单列表")
    public String listOrdersByClientInTime(
            @ToolParam(description = "客户编码，一般类似于'XX.XX.XXXX.XXX'格式，例如：01.01.0023.024。与clientName必须二选一", required = false) String clientCode,
            @ToolParam(description = "客户名称，一般类似于'某某课题组'或'某某公司'或'某某实验室'或'某某经营部'或'某某研究院'或'某某研究所'或'某某测试'等组织，例如：张三课题组，广州小懒科技有限公司，金域测试、金域课题组111、玉尘测试A01、睿展小组3号。与clientCode必须二选一", required = false) String clientName,
            @ToolParam(description = "开始时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String startTime,
            @ToolParam(description = "结束时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String endTime
    ) {
        JSONObject requestBody = new JSONObject();
        requestBody.put("clientCode", clientCode);
        requestBody.put("clientName", clientName);
        requestBody.put("startTime", startTime);
        requestBody.put("endTime", endTime);

        return executeRequest("ai/query/listOrdersByClientInTime", requestBody);
    }

    @Tool(description = "根据订单编码查询订单信息。orderCode是必填项，如果提取不到则输出：请告知需要查询的订单号")
    public JSONObject getOrderInfoByCode(
            @ToolParam(description = "订单编码（即订单号），值一般是由字母、数字和'-'组成，特征一般类似于'NJ00027202506090001'，或'NJ00027202506090001-G'或'NJ00027202506090001-T-1'，或'dev-NJ00027202506090001'") String orderCode
    ) {
        Assert.isTrue(StrUtil.isNotEmpty(orderCode), "订单编码不能为空");

        JSONObject requestBody = new JSONObject();
        requestBody.put("orderCode", orderCode);

        String executed = executeRequest("ai/query/getOrderInfoByCode", requestBody);
        return JSONObject.parseObject(executed);
    }

    @Tool(description = "根据合同编码查询下游订单编码列表。contractCode是必填项，如果提取不到则输出：请告知需要查询的合同编码")
    public String listOrdersByContractCode(
            @ToolParam(description = "合同编码，值一般是由字母、数字和'-'组成的合同标识符") String contractCode
    ) {
        Assert.isTrue(StrUtil.isNotEmpty(contractCode), "合同编码不能为空");

        JSONObject requestBody = new JSONObject();
        requestBody.put("conCode", contractCode);

        return executeRequest("ai/query/listOrdersByContractCode", requestBody);
    }

    @Tool(description = "根据订单编码、客户编码或客户名称查询出库单编码列表（支持时间区间）")
    public String listStockoutsByCondition(
            @ToolParam(description = "订单编码（即订单号），值一般是由字母、数字和'-'组成，特征一般类似于'NJ00027202506090001'，或'NJ00027202506090001-G'或'NJ00027202506090001-T-1'，或'dev-NJ00027202506090001'", required = false) String orderCode,
            @ToolParam(description = "客户编码，一般类似于'XX.XX.XXXX.XXX'格式，例如：01.01.0023.024", required = false) String clientCode,
            @ToolParam(description = "客户名称，一般类似于'某某课题组'或'某某公司'或'某某实验室'或'某某经营部'或'某某研究院'或'某某研究所'或'某某测试'等组织，例如：张三课题组，广州小懒科技有限公司，金域测试、金域课题组111、玉尘测试A01、睿展小组3号", required = false) String clientName,
            @ToolParam(description = "开始时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String startTime,
            @ToolParam(description = "结束时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String endTime
    ) {
        JSONObject requestBody = new JSONObject();
        requestBody.put("orderCode", orderCode);
        requestBody.put("clientCode", clientCode);
        requestBody.put("clientName", clientName);
        requestBody.put("startTime", startTime);
        requestBody.put("endTime", endTime);

        return executeRequest("ai/query/listStockoutsByCondition", requestBody);
    }

    @Tool(description = "根据订单编码、客户编码或客户名称查询退货单编码列表（支持时间区间）")
    public String listCancelOrdersByCondition(
            @ToolParam(description = "订单编码（即订单号），值一般是由字母、数字和'-'组成，特征一般类似于'NJ00027202506090001'，或'NJ00027202506090001-G'或'NJ00027202506090001-T-1'，或'dev-NJ00027202506090001'", required = false) String orderCode,
            @ToolParam(description = "客户编码，一般类似于'XX.XX.XXXX.XXX'格式，例如：01.01.0023.024", required = false) String clientCode,
            @ToolParam(description = "客户名称，一般类似于'某某课题组'或'某某公司'或'某某实验室'或'某某经营部'或'某某研究院'或'某某研究所'或'某某测试'等组织，例如：张三课题组，广州小懒科技有限公司，金域测试、金域课题组111、玉尘测试A01、睿展小组3号", required = false) String clientName,
            @ToolParam(description = "开始时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String startTime,
            @ToolParam(description = "结束时间，一般类似于'yyyy-MM-dd HH:mm:ss'格式，例如：2024-01-01 00:00:00。非必填", required = false) String endTime
    ) {
        JSONObject requestBody = new JSONObject();
        requestBody.put("orderCode", orderCode);
        requestBody.put("clientCode", clientCode);
        requestBody.put("clientName", clientName);
        requestBody.put("startTime", startTime);
        requestBody.put("endTime", endTime);

        return executeRequest("ai/query/listCancelOrdersByCondition", requestBody);
    }

    @Tool(description = "根据出库单编码查询出库单详情。stockoutCode是必填项，如果提取不到则输出：请告知需要查询的出库单编码")
    public String getStockoutDetail(
            @ToolParam(description = "出库单编码，值一般是由字母、数字和'-'组成的出库单标识符") String stockoutCode
    ) {
        Assert.isTrue(StrUtil.isNotEmpty(stockoutCode), "出库单编码不能为空");

        JSONObject requestBody = new JSONObject();
        requestBody.put("outCode", stockoutCode);

        return executeRequest("ai/query/getStockoutDetail", requestBody);
    }

    @Tool(description = "根据退货单编码查询退货单详情。cancelOrderCode是必填项，如果提取不到则输出：请告知需要查询的退货单编码")
    public String getCancelOrderDetail(
            @ToolParam(description = "退货单编码，值一般是由字母、数字和'-'组成的退货单标识符") String cancelOrderCode
    ) {
        Assert.isTrue(StrUtil.isNotEmpty(cancelOrderCode), "退货单编码不能为空");

        JSONObject requestBody = new JSONObject();
        requestBody.put("cancelCode", cancelOrderCode);

        return executeRequest("ai/query/getCancelOrderDetail", requestBody);
    }

    @Tool(description = "根据合同编码查询合同详情。contractCode是必填项，如果提取不到则输出：请告知需要查询的合同编码")
    public String getContractDetail(
            @ToolParam(description = "合同编码，值一般是由字母、数字和'-'组成的合同标识符") String contractCode
    ) {
        Assert.isTrue(StrUtil.isNotEmpty(contractCode), "合同编码不能为空");

        JSONObject requestBody = new JSONObject();
        requestBody.put("conCode", contractCode);

        return executeRequest("ai/query/getContractDetail", requestBody);
    }

    @Tool(description = "调用此函数以获取当前准确时间（包括日期和时区），用于代替模型默认推理")
    public String getCurrentTime() {
        return DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss");
    }


}
